#!/usr/local/bin/perl 

# ******************************************************
# ******************************************************
# Revision with added documentation 9/1998
# Revised by Barbara Mihalas 1996, 1997, 1998
# Written by Barbara Mihalas 1996
#
# For tab-delim files with 1st line = Field Names;
#
# reads multiple records with fields separated by
# $delim='\t' and determines the length of each field, then
# an array @maxlen stores the maximum length to date
# of each field
#
# Data input file is given as command line argument.
#
# Diagnostic printout is sent to file specified in
#   command line for running tab2db.pl 
# Command file (with command line) is run1. 
# run1 is:  
#          tab2db98.pl $1.tab > out/$1.out
# ******************************************************
# ******************************************************

#&test_argv;  #diagnostic; if needed

# **************************************************
# Set up parameters and filenames 
# **************************************************

$infile = $ARGV[0];
$fdelim = '\t';
$ndelim = '\&';

$basefilename = &get_basename($ARGV[0]);

&make_filenames($basefilename);

&print_filenames;    # diagnostic printout to screen

# **************************************************
# Process the input data file
# Determine length needed for data field
# Write the compacted output data file
# **************************************************
open (INDAT, $ARGV[0]) || die "can't open file $ARGV[0]\n";

open (NEWDAT, ">$datafilename") 
     || die "can't open file $datafilename for writing\n";   

$linecount = 0;
while (<INDAT>) 
{
   $linecount++;
   if ($linecount == 1 ) 
   {
      &get_fieldnames($_, $fdelim);
   }
   else 
   {
      next if (/^#/);
      next if (/^\n/);

    # *** if adding an index field to each row, use:
      &process_row_add_id($_, $fdelim, $linecount);

    # *** if NOT adding an index field to each row, use:
    # &process_row($_, $fdelim); 
   }
}

close(NEWDAT);
# **************************************************
# End of input data file processing
# **************************************************

# **************************************************
# diagnostic printout: command line for running 
# tab2db.pl redirects this to specified output file
# This is run1; we redirect to out/$infilestem.out
# **************************************************

print "\n\n";
&printlen;

# **************************************************
# write  sql & ctl files for loading table & data 
# **************************************************

#&write_tabledef;  #use if not adding an index field

&write_tabledef_add_id;  # use if adding an index field


# **************************************************
# **************************************************
sub test_argv {
   local ($i, $len, $argsub);
   foreach $i (0 .. $#ARGV) {
      $len = length($ARGV[$i]);
      print "$i $ARGV[$i] $len  -- ";
      $argsub = substr($ARGV[$i],0,index($ARGV[$i], '.') +1 );
      print "substr of argv is $argsub \n";
   }
}
# **************************************************
# **************************************************
sub get_basename {
   local ($name) = @_;
   local ($basename);
   $basename = substr($name, 0, index($name, '.') );
}
# **************************************************
#  Use the name of the input tab file to create names
#  for the output datafile and for the "create table"
#  sql file and the loader control file.
# **************************************************
sub make_filenames {
   local ($basename) = @_;
   $tablesqlname = $basename.'.sql';   
   $loadctlname  = $basename.'.load.ctl';   
   $datafilename = $basename.'.in.dat';   
   $tablename = $basename;   
}
# **************************************************
# **************************************************
sub print_filenames {
   print "basefilename of $ARGV[0] is $basefilename \n";
   print "\n";
   print "file to create table is $tablesqlname \n";
   print "control file to load data is $loadctlname \n";
   print "filename for processed data to load into table is $datafilename \n";
   print "tablename is $tablename \n";
}
# **************************************************
# **************************************************
sub get_fieldnames {
    local ($fnstring, $delim) = @_;
    print "\nfieldname string is\n $fnstring\n";
    $fnstring =~ s/\r\n//;
    print "\nfieldname string is\n $fnstring\n";
 	@fields = split (/$delim/, $fnstring); 
 	print +"FIELDS: @fields \n\n";
 	for ( $i=0; $i < @fields; $i++)  {
 		$fields[$i] =~ s/\"//g;
 		$fields[$i] =~ s/-/_/g;
 		$fields[$i] =~ s/\'//g;
        $fldlen[$i] = 17;
 		print +"Field $i is $fields[$i]; fieldlen set to $fldlen[$i]\n";
    }
}
# ***********************************************************
#  Processes a row and packs data with & as delimiter; adds
#  a string of &&&&& at the end of row to insure loading
#  if there are missing values at the end of the row.
# ***********************************************************
sub process_row {
   local($row,$delim) = @_;
   local($j);
   $row =~ s/\r\n//;
   $row =~ s/\n//;
   @record = split(/$delim/, $row);
   if ( $#record != $#fields) 
   {
      print +"record $recnum has $#record fields instead of $#fields\n";
      $#record <= $#fields || die "too many fields in record\n";
   }

# clean the field data and write to new datafile
#
   for ( $j=0; $j < @record; $j++)
   {
      @record[$j] =~ s/\"//g;       # remove quotes
      @record[$j] =~ s/\x1d/; /g;   # remove ^] chars
      @record[$j] =~ s/&/ and /g;   # replace ampersands with and
      @record[$j] =~ s/  and  / and /g;
    # write cleaned field data out to new data file with new delimiter
      print NEWDAT @record[$j], "&";
   }
   print NEWDAT "&&&&&&&&&\n";

   # set fieldlength to larger of current max or length of current field
   #
   for ($j=0; $j<@fields;$j++)
   {
      $len = length(@record[$j]);
      @fldlen[$j]= $len > @fldlen[$j] ? $len : @fldlen[$j];
   }
   $recnum++;
}
# ***********************************************************
#  Processes a row and packs data with & as delimiter; adds
#  a string of &&&&& at the end of row to insure loading
#  if there are missing values at the end of the row.
# 
#  This version adds a numerical (sequential) "record_id" 
#  that can be indexed to the start of each row.
# ***********************************************************
sub process_row_add_id 
{
   local($row,$delim, $record_id) = @_;
   local($j);
   $row =~ s/\r\n//;
   $row =~ s/\n//;
   @record = split(/$delim/, $row);
   if ( $#record != $#fields) 
   {
      print +"record $recnum has $#record fields instead of $#fields\n";
      $#record <= $#fields || die "too many fields in record\n";
   }

# clean the field data and write to new datafile
#
   print NEWDAT $record_id,"&";

   for ( $j=0; $j < @record; $j++)
   {
      @record[$j] =~ s/\"//g;       # remove quotes
      @record[$j] =~ s/\x1d/; /g;   # remove ^] chars
      @record[$j] =~ s/&/ and /g;   # replace ampersands with and
      @record[$j] =~ s/  and  / and /g; # remove extra spaces
   #  write cleaned field data out to new data file with new delimiter
      print NEWDAT @record[$j], "&";
   }
   print NEWDAT "&&&&&&&&&\n";

# set fieldlength to larger of current max or length of current field
#
   for ($j=0; $j<@fields;$j++) 
   {
      $len = length(@record[$j]);
      @fldlen[$j]= $len > @fldlen[$j] ? $len : @fldlen[$j];
   }
   $recnum++;
}

# ***********************************************************
# Diagnostic printout; not used except for debugging
# ***********************************************************
sub printdata {
    local(@data_array, @names, $printid) = @_;
    for ($j=0; $j<@data_array;$j++) {
       print TESTDAT "$printid of @names[$j] is @data_array[$j]\n";
#      print TESTDAT "@data_array[$j]\n";
    }
}
# ***********************************************************
# Diagnostic printout; not used except for debugging
# ***********************************************************
sub printlen 
{
   local ($i);
   print +"Final record lengths \n\n";
   for ($i=0; $i<@fields; $i++) 
   {
      #print +"Field @fields[$i] has final length @fldlen[$i]  \n";
      print +"Field $i is @fields[$i]; final value of fieldlen is @fldlen[$i]\n";
   }
   print +"\n";
}
# ***********************************************************
# Write disk file with the "create table" sql script.
# This version writes the sql for exactly the fields in the
# tab_delimited file.  
#
# Note, however, that the data type of the original data is
# not included in the tabfile, so the script may need to be
# edited to correctly identify numerical fields.
# ***********************************************************
sub write_tabledef {
   local ($i);
   open (TFILE, ">$tablesqlname") 
       || die "can't open $tablesqlname for writing\n";
   print TFILE "DROP TABLE $tablename;\n";
   print TFILE "CREATE TABLE $tablename (\n";

   open (CFILE, ">$loadctlname") ||
      die "can't open $loadctlname for writing\n";
   select (CFILE);
   print +"Load data\n";
   print +"infile '$datafilename'\n";
   print +"append\n";
   print +"into table $tablename\n";
   print +"fields terminated by '&'\n";
   print +"(\n";
#  print +"record_id,\n";
   select (STDOUT);

   for ($i=0; $i < $#fields; $i++) {
      $fldlen[$i] = int( 1.3 * $fldlen[$i] );
      print TFILE "$fields[$i] \t\t\t  varchar2($fldlen[$i]),\n";
      print CFILE "$fields[$i],\n";
   }
   $fldlen[$#fields] = int( 1.3 * $fldlen[$#fields] );
   print TFILE "$fields[$#fields] \t\t\t varchar2($fldlen[$#fields])\n";
   print TFILE ");\n";
   print TFILE "GRANT SELECT ON $tablename TO wow_user; \n";
   print TFILE "GRANT SELECT ON $tablename TO stg_user; \n";
   print TFILE "GRANT SELECT ON $tablename TO crr_user; \n";
   print TFILE "GRANT SELECT ON $tablename TO ns_user; \n";
   print CFILE "$fields[$#fields]\n";
   print CFILE ")\n";
   close TFILE;
   close CFILE;
}
# ***********************************************************
# Write disk file with the "create table" sql script.
# This version adds a numerical field "record_id"
# at the start; record_id is a unique id that can be indexed.
#
# Note, however, that the data type of the original data is
# not included in the tabfile, so the script may need to be
# edited to correctly identify numerical fields.
# ***********************************************************
sub write_tabledef_add_id {
   local ($i);

   # ***********************************************************
   # *** TFILE is the "CREATE TABLE" sql file ******************
   # *** CFILE is the oracle/unix control file for loading data
   # ** At least CFILE will need to be revised for Informix/NT
   # ***********************************************************

   open (TFILE, ">$tablesqlname") 
       || die "can't open $tablesqlname for writing\n";
   print TFILE "DROP TABLE $tablename;\n";
   print TFILE "CREATE TABLE $tablename (\n";
   print TFILE "record_id \t\t\t   number(10),\n";

   open (CFILE, ">$loadctlname") ||
      die "can't open $loadctlname for writing\n";
   select (CFILE);
   print +"Load data\n";
   print +"infile '$datafilename'\n";
   print +"append\n";
   print +"into table $tablename\n";
   print +"fields terminated by '&'\n";
   print +"(\n";
   print +"record_id,\n";
   select (STDOUT);

   for ($i=0; $i < $#fields; $i++) {
      $fldlen[$i] = int( 1.3 * $fldlen[$i] );
      print TFILE "$fields[$i] \t\t\t  varchar2($fldlen[$i]),\n";
      print CFILE "$fields[$i],\n";
   }
   $fldlen[$#fields] = int( 1.3 * $fldlen[$#fields] );
   print TFILE "$fields[$#fields] \t\t\t varchar2($fldlen[$#fields])\n";
   print TFILE ");\n";
   print TFILE "GRANT SELECT ON $tablename TO wow_user; \n";
   print TFILE "GRANT SELECT ON $tablename TO stg_user; \n";
   print TFILE "GRANT SELECT ON $tablename TO crr_user; \n";
   print TFILE "GRANT SELECT ON $tablename TO ns_user; \n";
   print CFILE "$fields[$#fields]\n";
   print CFILE ")\n";
   close TFILE;
   close CFILE;
}
# ***********************************************************
# ***********************************************************
