/* =============================================== */
DROP PACKAGE full_report;
CREATE OR REPLACE PACKAGE full_report AS

   PROCEDURE com_object_report;

   PROCEDURE display_mreport;

END full_report;
/
show errors;
/
/* =============================================== */

CREATE OR REPLACE PACKAGE BODY full_report IS

/* =============================================== */
   PROCEDURE com_object_report IS

   CURSOR get_models IS
   SELECT *
     FROM model_comprod
    ORDER BY model;

   CURSOR get_pp_fname (modelin IN VARCHAR2) IS
   SELECT * 
     FROM model_graphic
    WHERE upper(model) = upper(modelin)
      AND upper(usage) = 'PRODUCT SHOT';

   CURSOR get_ptxt_fname (modelin IN VARCHAR2) IS
   SELECT * 
     FROM model_text_pp
    WHERE upper(model) = upper(modelin)
      AND upper(t_usage) = 'PRODUCT DESCRIPTION';

   CURSOR get_segtxt_fname (modelin IN VARCHAR2) IS
   SELECT * 
     FROM model_text_pp
    WHERE upper(model) = upper(modelin)
      AND upper(t_usage) = 'SEGUE TO PRODUCT PAGE';

   CURSOR get_download_count (modelin IN VARCHAR2) IS
   SELECT count(*) 
     FROM model_download
    WHERE upper(model) = upper(modelin);

   mrec                get_models%ROWTYPE;
   pprec               get_pp_fname%ROWTYPE;
   ptrec               get_ptxt_fname%ROWTYPE;
   strec               get_segtxt_fname%ROWTYPE;
   cnum_downloads       NUMBER(10);
   cp_photo_fname       VARCHAR2(60);
   cp_txt_fname         VARCHAR2(60);
   cseg_txt_fname       VARCHAR2(60);
   mseq                 number(10);

   BEGIN


   DELETE FROM model_report;
   OPEN get_models;
   LOOP
      FETCH get_models INTO mrec;
      EXIT WHEN get_models%NOTFOUND;
      SELECT mrepseq.NEXTVAL INTO mseq FROM dual; 
      IF (mrec.datatablename IS NULL) THEN
         mrec.datatablename := 'none';
      END IF;
      INSERT INTO model_report
      (
        mrid, model, datatablename, division
      )
      VALUES 
      (
        mseq, mrec.model, mrec.datatablename, mrec.division
      );
      commit;
     htp.p('model_report: '||mseq||' '||mrec.model||' ');
     htp.p(' '||mrec.datatablename||' '||mrec.division||'>BR>');

     OPEN get_pp_fname (upper(mrec.model));
     FETCH get_pp_fname INTO pprec;
     IF (get_pp_fname%FOUND) THEN
        cp_photo_fname := pprec.g_filename||'.'||pprec.g_extension;
        UPDATE model_report SET
          p_photo_fname = cp_photo_fname
          where upper(model) = upper(mrec.model);
        htp.p('model_report graphic: '||cp_photo_fname||'<BR>');
     ELSE
        cp_photo_fname := 'none';
        UPDATE model_report SET
          p_photo_fname = cp_photo_fname
          where upper(model) = upper(mrec.model);
        htp.p('model_report graphic: '||cp_photo_fname||'<BR>');
     END IF;
     CLOSE get_pp_fname;
   
     OPEN get_ptxt_fname (upper(mrec.model) );
     FETCH get_ptxt_fname INTO ptrec;
     IF (get_ptxt_fname%FOUND) THEN
        cp_txt_fname := ptrec.t_filename||'.'||ptrec.t_extension;
        UPDATE model_report SET
          p_txt_fname = cp_txt_fname
          where upper(model) = upper(mrec.model);
        htp.p('model_report product description: '||cp_txt_fname||'<BR>');
     ELSE
        cp_txt_fname := 'none';
        UPDATE model_report SET
          p_txt_fname = cp_txt_fname
          where upper(model) = upper(mrec.model);
        htp.p('model_report product description: '||cp_txt_fname||'<BR>');
     END IF;
     CLOSE get_ptxt_fname;

     OPEN get_segtxt_fname (upper(mrec.model) );
     FETCH get_segtxt_fname INTO strec;
     IF (get_segtxt_fname%FOUND) THEN
        cseg_txt_fname := strec.t_filename||'.'||strec.t_extension;
        UPDATE model_report SET
          seg_txt_fname = cseg_txt_fname
          where upper(model) = upper(mrec.model);
        htp.p('model_report product segue: '||cseg_txt_fname||'<BR>');
     ELSE
        cseg_txt_fname := 'none';
        UPDATE model_report SET
          seg_txt_fname = cseg_txt_fname
          where upper(model) = upper(mrec.model);
        htp.p('model_report product segue: '||cseg_txt_fname||'<BR>');
     END IF;
     CLOSE get_segtxt_fname;

     OPEN get_download_count (upper(mrec.model) );
     FETCH get_download_count INTO cnum_downloads;
     IF (get_download_count%FOUND) THEN
        UPDATE model_report SET
          num_downloads = cnum_downloads
          where upper(model) = upper(mrec.model);
        htp.p('model report number of downloads: '||cnum_downloads||'<BR>');
     ELSE
        cnum_downloads := 0;
        UPDATE model_report SET
          num_downloads = cnum_downloads
          where upper(model) = upper(mrec.model);
        htp.p('model report number of downloads: '||cnum_downloads||'<BR>');
     END IF;
     CLOSE get_download_count;

   END LOOP;
   CLOSE get_models;

   END com_object_report;
/* =============================================== */
   PROCEDURE display_mreport IS

   CURSOR get_mreport IS
   SELECT model, datatablename, division, p_photo_fname, p_txt_fname, 
          seg_txt_fname, num_downloads
     FROM model_report
    ORDER BY model;

   mrec      get_mreport%ROWTYPE;

   BEGIN

   CCUTIL.chtml_header;

   htp.p('<TABLE BORDER=3><FONT SIZE=2>');
   htp.p('<TR>');
   htp.p('<TD><FONT SIZE=2> Model </FONT></TD>');
   htp.p('<TD><FONT SIZE=2> Data Table </FONT></TD>');
   htp.p('<TD><FONT SIZE=2> Div.</FONT></TD>');
   htp.p('<TD><FONT SIZE=2> Product Page<BR>Photo Filename </FONT></TD>');
   htp.p('<TD><FONT SIZE=2> Product Page<BR>Prod Description Filename</FONT></TD>');
   htp.p('<TD><FONT SIZE=2> Product Page<BR>Segue Text Filename</FONT></TD>');
   htp.p('<TD><FONT SIZE=2> Product Page<BR>Num Downloads</FONT></TD>');
   htp.p('</TR>');

   OPEN get_mreport;
   LOOP
      FETCH get_mreport INTO mrec;
      EXIT WHEN get_mreport%NOTFOUND;
      htp.p('<TR>');
      htp.p('<TD><FONT SIZE=2>'||mrec.model||'</FONT></TD>');
      htp.p('<TD><FONT SIZE=2>'||mrec.datatablename||'</FONT></TD>');
      htp.p('<TD><FONT SIZE=2>'||mrec.division||'</FONT></TD>');
      htp.p('<TD><FONT SIZE=2>'||mrec.p_photo_fname||'</FONT></TD>');
      htp.p('<TD><FONT SIZE=2>'||mrec.p_txt_fname||'</FONT></TD>');
      htp.p('<TD><FONT SIZE=2>'||mrec.seg_txt_fname||'</FONT></TD>');
      htp.p('<TD><FONT SIZE=2>'||mrec.num_downloads||'</FONT></TD>');
      htp.p('</TR>');
   END LOOP;
   htp.p('</FONT></TABLE>');
   CLOSE get_mreport;

   CCUTIL.chtml_footer;

   END display_mreport;
/* =============================================== */
END full_report;
/
show errors;
/
